@@ -52,14 +52,23 @@ import qualified Data.Text as T
 import qualified Data.Text.Array as TA
 import qualified Data.Text.Internal as T
 import qualified Data.Text.Lazy as LT
-import Foreign.C (CLong, CString)
+import Foreign.C (CString)
+#if __GLASGOW_HASKELL__ >= 703
+import Foreign.C (CLong(..))
+#else
+import Foreign.C (CLong)
+#endif
 import Foreign.Marshal.Utils (with)
 import Foreign.Ptr (Ptr, castPtr)
 import Foreign.Storable (alignment, peek, sizeOf)
 import System.IO.Unsafe (unsafePerformIO)
 
 #if defined(__GLASGOW_HASKELL__)
+# if __GLASGOW_HASKELL__ >= 703
+import Foreign.C.Types (CInt(..))
+# else
 import Foreign.C.Types (CInt)
+# endif
 import GHC.Base (ByteArray#)
 import GHC.Conc (ThreadId(..))
 import GHC.Prim (ThreadId#)
@@ -71,6 +80,12 @@ import Control.Concurrent (ThreadId)
 import System.Mem.StableName
 #endif
 
+import Data.Typeable
+#if __GLASGOW_HASKELL__ >= 702
+import GHC.Fingerprint.Type(Fingerprint(..))
+import Data.Typeable.Internal(TypeRep(..))
+#endif
+
 #include "MachDeps.h"
 
 infixl 0 `combine`, `hashWithSalt`
@@ -280,6 +295,24 @@ instance Hashable LT.Text where
     hash = hashWithSalt stringSalt
     hashWithSalt = LT.foldlChunks hashWithSalt
 
+
+-- | Compute the hash of a TypeRep, in various GHC versions we can do this quickly.
+hashTypeRep :: TypeRep -> Int
+{-# INLINE hashTypeRep #-}
+#if __GLASGOW_HASKELL__ >= 702
+-- Fingerprint is just the MD5, so taking any Int from it is fine
+hashTypeRep (TypeRep (Fingerprint x _) _ _) = fromIntegral x
+#elif __GLASGOW_HASKELL__ >= 606
+hashTypeRep = B.inlinePerformIO . typeRepKey
+#else
+hashTypeRep = hash . show
+#endif
+
+instance Hashable TypeRep where
+    hash = hashTypeRep
+    {-# INLINE hash #-}
+
+
 ------------------------------------------------------------------------
 -- * Creating new instances
 
